/*
Copyright (c) 2007-2013 Alloysoft, LLC. All rights reserved.
http://www.alloysoft.com
*/

var uiMonitorId = 0;
var loggingEnabled = true;
var isMobileSafari = true;
var isTouchCapable = false;
var isIPad = false;

var lastPlaylistPosition = -1;

var currentBrowsePath = "";
var currentSearchKeywords = "";
var currentArtURL = "";

var searchInput = "";
var searchInputTimer;

var lastPlaybackUpdateTime;
var lastPlaybackPostTime = new Date();
var playbackPosition = 0;
var lastPlaybackPosition = null;
var lastPostedPlaybackPosition = 0;
var lastMediaItemLength = 0;
var pendingPlaybackPostPosition = 0;
var playbackTimerId = 0;
var playbackPostTimerId = 0;
var isPlaying = false;

// UI control
var showingMediaLib = false;
var showingPlaylist = false;
var showingSpeakers = false;
var showingSearchResults = false;
var showingMediaLibSearch = false;
var showingMediaLibActions = false;
var showingMediaLibAlphaJump = false;
var showingMediaLibSearchResults = false;
var showingSubControls = false;
var timeElapsedElem;
var timeRemainingElem;

var itemsPerPage = 35;
var volumeIncrement = 10;
var slideSpeed = 4.0;

var mediaLibStart = 0;
var mediaLibLength = 0;
var playlistStart = 0;
var playlistLength = 0;
var showPlaylistItemOnPageLoad = false;
var browseHistory = new Array();
var currentActionItem = "";
var useMultipleSpeakers = false;

var albumArtCurrentName = "albumart";
var albumArtNewName = "albumart2";

var playbackSlider;
var volumeSlider;

// history
var navHistory = new Array();
var navHash = '#';
var navCurrent = '';

BrowseHistoryEntry = function()
{
	this.path = "";
	this.search = false;
	this.start = 0;
}

function onConnectedToPlayer() {
	player.browseLibrary(currentBrowsePath, mediaLibStart, itemsPerPage);
	player.getPlaylistItems(0, itemsPerPage);

	timeElapsedElem = document.getElementById("time_elapsed");
	timeRemainingElem = document.getElementById("time_remaining");
}

function onCurrentMediaChanged(mediaItem) {
	var elem = document.getElementById('artist');
	elem.innerHTML = mediaItem.artist;

	elem = document.getElementById('album');
	elem.innerHTML = mediaItem.album;

	elem = document.getElementById('title');
	elem.innerHTML = mediaItem.title;

	if (mediaItem.length != lastMediaItemLength) {
		updatePlaybackPosition(playbackPosition);
		lastPlaybackUpdateTime = new Date();
	}
}

function onPlayingStateChanged(playingState) {
	var image = '<img src="images/ctrl_play.png">';
	if (playingState == 'playing') {
		image = '<img src="images/ctrl_pause.png">';
	}

	message = document.getElementById('ctrl_play');
	message.innerHTML = image;

	var playing = (playingState == "playing");

	if (isPlaying != playing) {
		if (playing) {
			isPlaying = true;
			lastPlaybackUpdateTime = new Date();
			playbackTimerId = setInterval(incrementPlaybackPosition, 500);
		} else {
			isPlaying = false;
			clearInterval(playbackTimerId);
			playbackTimerId = 0;
		}
	}
}

function onVolumeChanged(volume) {
	volumeSlider.setPos(volume / 100.0);
}

function onPlaylistChanged() {
	// refresh the current playlist view
	player.getPlaylistItems(playlistStart, itemsPerPage);
}

function onPlaylistPositionChanged(playlistPosition) {
	var playlistItem = document.getElementById('playlist_' + lastPlaylistPosition);
	if (playlistItem) {
		playlistItem.className = "";
	}

	playlistItem = document.getElementById('playlist_' + playlistPosition);
	if (playlistItem) {
		playlistItem.className = "playing";
	}

	lastPlaylistPosition = playlistPosition;

	// if we are not showing the playlist, determine which
	// page the current media item is on and ensure it is visible
	if (!showingPlaylist) {
		if (playlistPosition < playlistStart || playlistPosition > (playlistStart + itemsPerPage)) {
			// determine which page needs to be displayed
			var page = parseInt(playlistPosition / itemsPerPage);
			player.getPlaylistItems(page * itemsPerPage, itemsPerPage);
		}
	}
}

function onPlaybackPositionChanged(position) {
	if (!playbackSlider.isDragging) {
		updatePlaybackPosition(position * 1000);
		lastPlaybackUpdateTime = new Date();
	}
}

function updatePlaybackPosition(position, isDrag) {
	var currentMediaLength = player.currentMedia.length;
	playbackPosition = position;
	position = parseInt(position / 1000);
	if (position > currentMediaLength) {
		position = currentMediaLength;
	}

	if (position !== lastPlaybackPosition || currentMediaLength != lastMediaItemLength) {
		lastPlaybackPosition = position;
		lastMediaItemLength = currentMediaLength;
		var remaining = currentMediaLength - position;
		if (remaining < 0) {
			remaining = 0;
		}

		timeElapsedElem.innerHTML = formatTime(position);
		timeRemainingElem.innerHTML = '-' + formatTime(remaining);

		if (!isDrag) {
			playbackSlider.setPos(currentMediaLength > 0 ? (position / currentMediaLength) : 0);
		}
	}
}

function incrementPlaybackPosition() {
	var now = new Date();
	var elapsed = now - lastPlaybackUpdateTime;
	updatePlaybackPosition(playbackPosition + elapsed);
	lastPlaybackUpdateTime = now;
}

function formatTime(time) {
	var result = '';
	var minutesLeading = 1;

	var hours = parseInt(time / 3600);
	time -= hours * 3600;
	var minutes = parseInt(time / 60);
	time -= minutes * 60;

	if (hours > 0) {
		result += hours + ':';
		minutesLeading = 2;
	}

	result += addLeadingZeros(minutes, minutesLeading) + ':' + addLeadingZeros(time, 2);
	return result;
}

function addLeadingZeros(num, length) {
	var result = '' + num;
	while (result.length < length) {
		result = '0' + result;
	}

	return result;
}

function onShuffleStateChanged(shuffleState) {
	var shuffleElem = document.getElementById("shufflecontrol");
	if (shuffleElem) {
		if (shuffleState) {
			shuffleElem.style.background = 'url("images/shuffle_on.png") center no-repeat';
		} else {
			shuffleElem.style.background = 'url("images/shuffle_off.png") center no-repeat';
		}
	}
}

function onRepeatStateChanged(repeatState) {
	var repeatElem = document.getElementById("repeatcontrol");
	if (repeatElem) {
		if (repeatState) {
			repeatElem.style.background = 'url("images/repeat_on.png") center no-repeat';
		} else {
			repeatElem.style.background = 'url("images/repeat_off.png") center no-repeat';
		}
	}
}

function onSpeakerListChanged(speakers) {
	var controlElem = document.getElementById("speakercontrolcontainer");
	var speakersElem = document.getElementById("speakerlist");
	var list = "";

	for (var i=0; i < speakers.length; i++) {
		var speaker = speakers[i];
		list += '<li onclick="handleSpeakerSelection(' + speaker.id + ')">' + speaker.name;
		if (speaker.enabled) {
			list += '<div class="activespeaker">&#10003;</div>';
		}
		list += '</li>';
	}

	speakersElem.innerHTML = list;

	if (speakers.length > 1) {
		controlElem.style.display = 'block';
	} else {
		controlElem.style.display = 'none';
	}
}

function onMediaLibraryChanged() {
	// refresh the current media library view
	player.browseLibrary(currentBrowsePath, 0, itemsPerPage);
}

function onResponseError(errorCode) {
	if (errorCode == PlayerError.UIScriptingDisabled) {
		alert('Please check "Enable access for assistive devices" on your Mac to use this feature.');
	}
}

function onBrowseLibraryResponse(response) {
	var list = "<ul>";
	var item;
	var type = "";
	var imgFile = "";
	var isFolder = false;
	var showAddOption = false;
	var newPath = "";

	if (showingMediaLibAlphaJump)
		showMediaLibAlphaJump();

	if (showingMediaLibSearch) {
		showMediaLibSearch();
	}

	if (showingMediaLibSearchResults) {
		showingMediaLibSearchResults = false;
	}

	if (showingMediaLib) {
		hideAddressBar();
	}

	showingSearchResults = false;

	currentBrowsePath = response.path;

	// identify the name of the current path
	var name = 'Media Library';
	if (currentBrowsePath.length > 1) {
		var pos = currentBrowsePath.lastIndexOf('/');
		if (pos != -1) {
			pos++;
			name = decodeURIComponent(currentBrowsePath.substr(pos, currentBrowsePath.length - pos));
			if (name.length > 0) {
				if (name.charAt(name.length - 1) == '/') {
					name = name.substr(0, name.length - 1);
				}
			} else {
				name = 'Media Library';
			}
		}
	}

	name = escapeHtmlEntities(name);
	if (document.getElementById) {
		message = document.getElementById('libheadertitle');
		message.innerHTML = name;
	} else {
		libheadertitle.innerHTML = name;
	}

	if (response.start !== null) {
		mediaLibStart = response.start;
	}

	if (response.totalCount !== null && response.totalCount >= 0) {
		mediaLibLength = response.totalCount;
	}

	// add the page controls if necessary
	if (mediaLibLength > itemsPerPage) {
		list += '<li>';

		if (mediaLibStart > 0) {
			list += '<div id="navprev" onclick="mediaLibPrevPage()">&lt; Prev</div>';
		} else {
			list += '<div id="navprev">&nbsp;</div>';
		}

		list += '<div id="navalphajump" onclick="showMediaLibAlphaJump()">A-Z</div>';

		if ((mediaLibStart + itemsPerPage) < mediaLibLength) {
			list += '<div id="navnext" onclick="mediaLibNextPage()">Next &gt;</div>';
		} else {
			list += '<div id="navnext">&nbsp;</div>';
		}

		list += '</li>';
	}

	for (var i=0, length = response.items.length; i < length; i++) {
		item = response.items[i];
		type = item.type;
		isFolder = false;
		showAddOption = true;

		if (item.type == 'folder') {
			isFolder = true;
		}

		var name = item.title;

		if (isFolder) {
			if (name == 'Music') {
				imgFile = 'cd';
			} else if (name == 'Videos') {
				imgFile = 'video';
			} else if (name == 'TV') {
				imgFile = 'tv';
			} else if (name == 'Genres') {
				imgFile = 'genre';
			} else if (name == 'Artists') {
				imgFile = 'artist';
			} else if (name == 'Albums') {
				imgFile = 'album';
			} else if (name == 'Playlists') {
				imgFile = 'playlist';
			} else if (name == 'Songs') {
				imgFile = 'music';
			} else if (name == 'Podcasts') {
				imgFile = 'podcast';
			} else if (name == 'Radio') {
				imgFile = 'podcast';
			} else {
				imgFile = 'music';
			}
		} else {
			if (type == 'genre') {
				imgFile = 'genre';
			} else 	if (type == 'artist') {
				imgFile = 'artist';
			} else 	if (type == 'album') {
				imgFile = 'album';
			} else if (type == 'podcast_show') {
				imgFile = 'podcast';
			} else if (type == 'tv_series') {
				imgFile = 'artist';
				showAddOption = false;
			} else 	if (type == 'playlist') {
				imgFile = 'playlist';
			} else 	if (type == 'playlist_folder' || type == 'radio_folder') {
				imgFile = 'playlist_folder';
				showAddOption = false;
			} else 	if (type == 'info') {
				imgFile = 'info';
			} else {
				imgFile = 'music';
			}
		}

		newPath = currentBrowsePath + "/" + encodeURIComponent(name);
		newPath = newPath.replace(/'/gmi, "\\'");

		list += '<li id="medialib_' + (mediaLibStart + i) + '" ';

		if (item.container) {
			if (!isFolder && showAddOption) {
				list += 'class="group" '
			}
		} else if (type == "music" || type == "podcast") {
			list += '><div id="grouptitle"';
		}

		if (!item.container) {
			//if (item.type == 'music' && currentBrowsePath.indexOf('Music/Songs') == -1) {
			//	list += 'onclick="handlePlayViewFromItem(this, ' + item.mediaId + ')"';
			//} else {
				list += 'onclick="handlePlayItem(this, ' + item.mediaId + ')"';
			//}
		} else {
			list += 'onclick="handleBrowseLibrary(this, \'' + newPath + '\')"';
		}

		list += '>';

		if (imgFile != '') {
		 list += '<img src="images/' + imgFile + '.png">';
		}

		list += name;

		if (item.container) {
			if (!isFolder && showAddOption) {
				list += '<div id="showactions" onclick="showMediaLibActions(\'' + newPath + '\')"></div>';
			}
		} else if (type == "music" || type == "podcast") {
			list += '</div><div id="showactions" onclick="handleSingleItemAdd(this, ' + item.mediaId + ')"></div>';
		}

		list += '</li>';
	}

	// add the page controls if necessary
	if (mediaLibLength > itemsPerPage) {
		list += '<li>';

		if (mediaLibStart > 0) {
			list += '<div id="navprev" onclick="mediaLibPrevPage()">&lt; Prev</div>';
		} else {
			list += '<div id="navprev">&nbsp;</div>';
		}

		list += '<div id="navalphajump" onclick="showMediaLibAlphaJump()">A-Z</div>';

		if ((mediaLibStart + itemsPerPage) < mediaLibLength) {
			list += '<div id="navnext" onclick="mediaLibNextPage()">Next &gt;</div>';
		} else {
			list += '<div id="navnext">&nbsp;</div>';
		}

		list += '</li>';
	}

	list += "</ul>";

	if (document.getElementById)
	{
		message = document.getElementById('libcontents');
		message.innerHTML = list;
	}

	// if we have received an alpha start position in response to a
	// browse starting from a particular letter, scroll to the result
	// if it is beyond the initial screen
	if (response.scrollToIndex != -1 && showingMediaLib)
	{
		var mediaLibItem = document.getElementById('medialib_' + response.scrollToIndex);
		if (mediaLibItem) {
			slideTo(mediaLibItem.offsetTop);
		}
	}
}

function onSearchLibraryResponse(response) {
	var list = "<ul>";
	var result;
	var newPath = "";

	// backup scrolling to work around Safari displaying address bar
	// when the keyboard is shown
	//hideAddressBar();

	if (!showingMediaLibSearchResults)
	{
		addBrowseHistoryEntry();
		showingMediaLibSearchResults = true;

		if (document.getElementById)
		{
			message = document.getElementById('libheadertitle');
			message.innerHTML = 'Search';
		}
		else
		{
			libheadertitle.innerHTML = 'Search';
		}
	}

	// retrieve the new start position
	mediaLibStart = response.start;

	// retrieve the total count if included
	if (response.totalCount !== null && response.totalCount >= 0) {
		mediaLibLength = response.totalCount;
	}

	for (var i=0, length = response.items.length; i < length; i++)
	{
		item = response.items[i];
		var type = item.type;
		var itemClass = "music";
		var imgFile = "music";
		newPath = "";

		itemClass = item.type;
		if (itemClass == 'podcast_show') {
			itemClass = 'podcast';
		} else if (itemClass == 'podcast') {
			itemClass = 'music';
		}

		list += '<li';

		if (!item.container)
		{
			if (type != 'playlist') {
				list += '><div id="grouptitle" onclick="handlePlayItem(this, ' + item.mediaId + ')"';
			} else {
				list += ' onclick="handlePlayItem(this, ' + item.mediaId + ')"';
			}
		}
		else
		{
			newPath = "/Music/";

			if (type == "genre")
				newPath += "Genres/";
			else if (type == "artist")
				newPath += "Artists/";
			else if (type == "album")
				newPath += "Albums/";
			else if (type == "podcast_show")
				newPath = "/Podcasts/";

			newPath += encodeURIComponent(item.title);
			newPath = newPath.replace(/'/gmi, "\\'");

			list += ' class="group" onclick="handleBrowseLibrary(this, \'' + newPath + '\')"';
		}

		list += '><img src="images/' + itemClass + '.png">' + item.title;

		if (item.container) {
			list += '<div id="showactions" onclick="showMediaLibActions(\'' + newPath + '\')"></div>';
		} else {
			if (type != 'playlist') {
				list += '</div><div id="showactions" onclick="handleSingleItemAdd(this, ' + item.mediaId + ')"></div>';
			}
		}

		list += '</li>';
	}

	list += "</ul>";

	if (document.getElementById)
	{
		message = document.getElementById('libcontents');
		message.innerHTML = list;
	}
	else
	{
		libcontents.innerHTML = list;
	}

	showingSearchResults = true;
}

function onGetPlaylistItemsResponse(response) {
	var itemIndex = 0;
	var list = '<ul>';

	if (showingPlaylist) {
		hideAddressBar();
	}

	playlistStart = response.start;

	if (response.totalCount !== null && response.totalCount >= 0) {
		playlistLength = response.totalCount;
	}

	// add the page controls if necessary
	if (playlistLength > itemsPerPage) {
		list += '<li>';

		if (playlistStart > 0) {
			list += '<div id="navprev" onclick="playlistPrevPage()">&lt; Prev</div>';
		}

		if ((playlistStart + itemsPerPage) < playlistLength) {
			list += '<div id="navnext" onclick="playlistNextPage()">Next &gt;</div>';
		}

		list += '</li>';
	}

	for (i=0, length = response.items.length; i < length; i++) {
		var item = response.items[i];
		itemIndex = playlistStart + i;
		list += '<li id="playlist_' + itemIndex + '" onclick="handlePlaylistJump(this, ' + itemIndex + ')"';

		if (itemIndex == player.playlistPosition) {
			list += 'class="playing" ';
		}

		list += '>' + item.title + ' <span class="listArtist">' + item.artist + '</span></li>';
	}

	if (response.items.length < 1) {
		list += '<li>No items</li>';
	}

	// add the page controls if necessary
	if (playlistLength > itemsPerPage) {
		list += '<li>';

		if (playlistStart > 0) {
			list += '<div id="navprev" onclick="playlistPrevPage()">&lt; Prev</div>';
		}

		if ((playlistStart + itemsPerPage) < playlistLength) {
			list += '<div id="navnext" onclick="playlistNextPage()">Next &gt;</div>';
		}

		list += '</li>';
	}

	list += '</ul>';

	if (document.getElementById)
	{
		message = document.getElementById('playlistcontents');
		message.innerHTML = list;
	}
	else
	{
		playlist.innerHTML = list;
	}

	// show the currently playing item, if requested
	if (showPlaylistItemOnPageLoad) {
		showPlaylistItemOnPageLoad = false

		if (player.playlistPosition >= playlistStart && player.playlistPosition < (playlistStart + itemsPerPage)) {
			showCurrentPlaylistItem();
		}
	}
}

function requestInitialState()
{
	sendRequest('<GetCurrentStateRequest></GetCurrentStateRequest>');
	sendRequest('<GetSpeakerListRequest></GetSpeakerListRequest>');
	player.browseLibrary(currentBrowsePath, mediaLibStart, itemsPerPage);
	player.getPlaylistItems(0, itemsPerPage);
}

function testForMobileSafari()
{
	try
	{
		if (navigator.userAgent)
		{
			var pos = navigator.userAgent.indexOf(' Mobile/');
			if (pos != -1)
			{
				pos = navigator.userAgent.indexOf(' Safari/');
				if (pos != -1)
				{
					pos = navigator.userAgent.indexOf('iPad');
					if (pos != -1)
					{
						isIPad = true;
						return false;
					}

					return true;
				}
			}
		}
	}
	catch (e) {}

	return false;
}

function onLoad()
{
	isMobileSafari = testForMobileSafari();
	player.navigatorOnlineEnabled = isMobileSafari;

	isTouchCapable = ("ontouchstart" in document.documentElement);

	// Adjust the album art size based on the screen's pixel ratio
	if (window.devicePixelRatio) {
		player.albumArtSize = player.albumArtSize * parseInt(window.devicePixelRatio);
	}

	// ensure that we display Now Playing despite any initial hash on load
	if (isMobileSafari && location.hash != navCurrent && location.hash != 'NowPlaying')
	{
		navCurrent = 'NowPlaying';
		location.hash = navCurrent;
	}

	// immediately apply any appropriate UI modifications
	uiMonitor();

	// add an event listener for animations
	var element = document.getElementById(albumArtCurrentName);
	if (element && element.addEventListener) {
		element.addEventListener('webkitTransitionEnd', onAlbumArtAnimationEnd, false);
	}

	if (document.body.addEventListener) {
		document.body.addEventListener('offline', function() {
			updateOnlineState('offline');
		}, false);
		document.body.addEventListener('online', function() {
			updateOnlineState('online');
		}, false);
	}

	playbackSlider = new Slider('scrubbercontrol', function(progress) {
		var now = new Date();
		var newPos = parseInt(lastMediaItemLength * progress);
		if (newPos != lastPlaybackPosition) {
			updatePlaybackPosition(newPos * 1000, true);
			if (now - lastPlaybackPostTime > 500) {
				if (playbackPostTimerId != 0) {
					clearTimeout(playbackPostTimerId);
					playbackPostTimerId = 0;
				}
				player.setPlaybackPosition(newPos);
				lastPostedPlaybackPosition = newPos;
				lastPlaybackPostTime = now;
				lastPlaybackUpdateTime = now;
			} else {
				pendingPlaybackPostPosition = newPos;
				if (playbackPostTimerId == 0) {
					playbackPostTimerId = setTimeout(function() {
						playbackPostTimerId = 0;
						if (Math.abs(pendingPlaybackPostPosition - lastPostedPlaybackPosition) > 10) {
							player.setPlaybackPosition(pendingPlaybackPostPosition);
							lastPostedPlaybackPosition = pendingPlaybackPostPosition;
							lastPlaybackPostTime = new Date();
							lastPlaybackUpdateTime = lastPlaybackPostTime;
						}
					}, 500);
				}
			}
		}
	});

	volumeSlider = new Slider('volumecontrol', function(progress) {
		var newVolume = parseInt(progress * 100.0);
		if (newVolume != player.volume) {
			player.setVolume(newVolume);
		}
	});

	player.onConnectedToPlayer = onConnectedToPlayer;
	player.onCurrentMediaChanged = onCurrentMediaChanged;
	player.onAlbumArtChanged = onAlbumArtChanged;
	player.onRatingChanged = onRatingChanged;
	player.onPlayingStateChanged = onPlayingStateChanged;
	player.onShuffleStateChanged = onShuffleStateChanged;
	player.onRepeatStateChanged = onRepeatStateChanged;
	player.onVolumeChanged = onVolumeChanged;
	player.onPlaylistChanged = onPlaylistChanged;
	player.onPlaylistPositionChanged = onPlaylistPositionChanged;
	player.onPlaybackPositionChanged = onPlaybackPositionChanged;
	player.onSpeakerListChanged = onSpeakerListChanged;
	player.onResponseError = onResponseError;
	player.onMediaLibraryChanged = onMediaLibraryChanged;
	player.onBrowseLibraryResponse = onBrowseLibraryResponse;
	player.onSearchLibraryResponse = onSearchLibraryResponse;
	player.onGetPlaylistItemsResponse = onGetPlaylistItemsResponse;

	setTimeout(setupApp, 100);
}

function updateOnlineState(state) {
	if (isMobileSafari) {
		if (state == 'offline') {
			if (uiMonitorId) {
				clearInterval(uiMonitorId);
				uiMonitorId = 0;
			}
		} else {
			uiMonitorId = setInterval(uiMonitor, 300);
		}
	}
}

function setupApp() {
	hideAddressBar();
	player.start();

	if (isMobileSafari) {
		uiMonitorId = setInterval(uiMonitor, 300);
	}

	// read cookie settings
	var val = getCookie("useMultipleSpeakers");
	if (val != null && val == "true" && !useMultipleSpeakers) {
		toggleUseMultipleSpeakers();
	}
}

function Slider(basename, onDrag) {
	var self = this;
	this.lastEventPos = 0;
	this.isDragging = false;
	this.onDrag = onDrag;

	this.slider = document.getElementById(basename + '_slider');
	this.overlay = document.getElementById(basename + '_on');
	this.base = document.getElementById(basename);

	this.currentX = 0;
	this.sliderOffset = 6;
	this.maxX = this.overlay.offsetWidth - this.sliderOffset;

	this.setPos = function(progress) {
		if (!this.isDragging) {
			this.moveTo(parseInt(this.maxX * progress));
		}
	};

	this.moveTo = function(newX) {
		var adjust = 0;
		if (newX < 0) {
			adjust = newX;
			newX = 0;
		} else if (newX > this.maxX) {
			adjust = newX - this.maxX;
			newX = this.maxX;
		}
		this.slider.style.left = (newX - this.sliderOffset) + 'px';
		this.overlay.style.width = newX + "px";
		this.currentX = newX;
		return adjust;
	};

	this.onTouchStart = function(e) {
		if (e.touches.length != 1) {
			return false;
		}

		this.isDragging = true;
		this.lastEventPos = e.touches[0].clientX;
		this.slider.addEventListener("touchmove", this, false);
		this.slider.addEventListener("touchend", this, false);
	};

	this.onTouchEnd = function(e) {
		this.isDragging = false;
		this.slider.removeEventListener("touchmove", this, false);
		this.slider.removeEventListener("touchend", this, false);
	};

	this.onMouseDown = function(e) {
		this.isDragging = true;
		this.lastEventPos = e.clientX;
		window.addEventListener("mousemove", this, false);
		window.addEventListener("mouseup", this, false);
		this.base.removeEventListener("click", this, false);
		this.overlay.removeEventListener("click", this, false);
	};

	this.onMouseUp = function(e) {
		this.isDragging = false;
		window.removeEventListener("mousemove", this, false);
		window.removeEventListener("mouseup", this, false);

		if (!isTouchCapable && e.offsetX !== undefined) {
			setTimeout(function() {
				self.base.addEventListener("click", self, false);
				self.overlay.addEventListener("click", self, false);
			}, 1);
		}
	};

	this.onMove = function(e) {
		var x = e.touches !== undefined ? e.touches[0].clientX : e.clientX;
		var adjust = this.moveTo(this.currentX + (x - this.lastEventPos));
		this.lastEventPos = x - adjust;
		if (this.onDrag) {
			this.onDrag(this.currentX / this.maxX);
		}
	};

	this.onClick = function(e) {
		if (e.offsetX === undefined) {
			return;
		}

		var x = e.offsetX;
		this.moveTo(x);
		if (this.onDrag) {
			this.onDrag(this.currentX / this.maxX);
		}
	};

	this.handleEvent = function(e) {
		switch(e.type) {
			case 'touchstart':
				this.onTouchStart(e);
				break;
			case 'touchmove':
				this.onMove(e);
				break;
			case 'touchend':
				this.onTouchEnd(e);
				break;

			case 'mousedown':
				this.onMouseDown(e);
				break;
			case 'mousemove':
				this.onMove(e);
				break;
			case 'mouseup':
				this.onMouseUp(e);
				break;

			case 'click':
				this.onClick(e);
				break;
			default:
				return;
		}

		if (e.preventDefault) {
			e.preventDefault();
		} else {
			e.returnValue = false;
		}
	};

	this.handleEventIE = function(e) {
		self.handleEvent.call(self, e);
	};

	this.addEventListener = function(obj, event) {
		if (obj.addEventListener) {
			obj.addEventListener(event, this, false);
		} else if (obj.attachEvent) {
			obj.attachEvent("on" + event, this.handleEventIE);
		}
	};

	if (this.slider.addEventListener) {
		this.slider.addEventListener("touchstart", this, false);
		this.slider.addEventListener("mousedown", this, false);
	}

	if (!isTouchCapable) {
		// Handle IE events
		this.addEventListener(this.base, "click");
		this.addEventListener(this.overlay, "click");
	}

	this.setPos(0);
}

function hideAddressBar() {
	window.scrollTo(0,1);
}

function backToNowPlaying() {
	/*if (isMobileSafari)
	{
		// use browser history to navigate
		if (navHistory.length > 0)
		{
			history.back();
		}
		else
		{
			setLocation('', false);
		}
	}*/

	// just hide the appropriate interface
	if (showingMediaLib)
	{
		showMediaLib(false);
	}
	else if (showingPlaylist)
	{
		showPlaylist(false);
	}
	else if (showingSpeakers)
	{
		showSpeakers(false);
	}
}

function showMediaLib(recordHistory) {
	if (document.getElementById)
	{
		hideAddressBar();
		//browseHistory.splice(0, browseHistory.length);

		var medialib = document.getElementById('medialib');
		var container = document.getElementById('container');

		if (!showingMediaLib) {

			/*if ((currentBrowsePath != "" && currentBrowsePath != "/") || (showingSearchResults)) {
				browseLibrary("", 0, itemsPerPage);
			}*/

			setLocation('Media', recordHistory);

			if (showingPlaylist) {
				showPlaylist();
			}

			showingMediaLib = true;
			medialib.style.overflow = 'visible';
			medialib.style.visibility = 'visible';
			container.style.overflow = 'visible';
		} else {
			if (showingMediaLibSearch) {
				showMediaLibSearch();
			}

			if (showingMediaLibAlphaJump)
				showMediaLibAlphaJump();

			showingMediaLib = false;
			medialib.style.overflow = 'hidden';
			container.style.overflow = 'hidden';

			// work around an iPad display bug
			if (isIPad) {
				setTimeout(function() { medialib.style.visibility = 'hidden'; }, 0);
			} else {
				medialib.style.visibility = 'hidden';
			}
		}
	}
}

function showPlaylist(recordHistory) {
	if (document.getElementById)
	{
		var playlist = document.getElementById('playlist');
		var container = document.getElementById('container');

		if (!showingPlaylist) {

			setLocation('Playlist', recordHistory);

			if (showingMediaLib) {
				showMediaLib(false);
			}

			hideAddressBar();
			playlist.style.overflow = 'visible';
			playlist.style.visibility = 'visible';
			container.style.overflow = 'visible';
			showingPlaylist = true;
		} else {
			playlist.style.overflow = 'hidden';
			container.style.overflow = 'hidden';

			// work around an iPad display bug
			if (isIPad) {
				setTimeout(function() { playlist.style.visibility = 'hidden'; }, 0);
			} else {
				playlist.style.visibility = 'hidden';
			}

			showingPlaylist = false;
			hideAddressBar();
		}
	}
}

function showSpeakers(recordHistory) {
	if (document.getElementById)
	{
		var speakers = document.getElementById('speakers');
		var container = document.getElementById('container');

		if (!showingSpeakers) {

			setLocation('Speakers', recordHistory);

			hideAddressBar();
			speakers.style.overflow = 'visible';
			speakers.style.visibility = 'visible';
			container.style.overflow = 'visible';
			showingSpeakers = true;
		} else {
			speakers.style.visibility = 'hidden';
			speakers.style.overflow = 'hidden';
			container.style.overflow = 'hidden';

			// work around an iPad display bug
			if (isIPad) {
				setTimeout(function() { speakers.style.visibility = 'hidden'; }, 0);
			} else {
				speakers.style.visibility = 'hidden';
			}

			showingSpeakers = false;
			hideAddressBar();
		}
	}
}

function showMediaLibSearch() {

	if (showingMediaLibAlphaJump)
		showMediaLibAlphaJump();

	if (document.getElementById)
	{
		var searchHeader = document.getElementById('libsearchfield');

		if (!showingMediaLibSearch) {
			searchHeader.style.visibility = 'visible';

			var searchField = document.getElementById('searchInput');
			if (searchField) {
				searchField.focus();
			}

			searchInputTimer = setInterval(searchInputMonitor, 300);

			showingMediaLibSearch = true;
		} else {
			searchHeader.style.visibility = 'hidden';

			var searchField = document.getElementById('searchInput');
			searchField.value = '';

			if (searchInputTimer) {
				clearInterval(searchInputTimer);
				searchInputTimer = null;
			}

			showingMediaLibSearch = false;
		}
	}
}

function showMediaLibActions(path) {
	if (document.getElementById)
	{
		var itemActions = document.getElementById('itemactions');
		currentActionItem = path;

		if (!showingMediaLibActions) {
			hideAddressBar();
			itemActions.style.visibility = 'visible';
			showingMediaLibActions = true;
		} else {
			itemActions.style.visibility = 'hidden';
			showingMediaLibActions = false;
			hideAddressBar();
		}
	}
}

function showMediaLibAlphaJump() {
	if (document.getElementById)
	{
		var alphaJumpElem = document.getElementById('alphajump');

		if (!showingMediaLibAlphaJump) {
			hideAddressBar();
			alphaJumpElem.style.display = 'block';
			alphaJumpElem.style.visibility = 'visible';
			showingMediaLibAlphaJump = true;
		} else {
			alphaJumpElem.style.visibility = 'hidden';
			alphaJumpElem.style.display = 'none';
			showingMediaLibAlphaJump = false;
			hideAddressBar();
		}
	}
}

function showCurrentPlaylistItem() {
	// first ensure that the page containing this item is loaded
	if (player.playlistPosition < playlistStart || player.playlistPosition >= (playlistStart + itemsPerPage)) {
		// determine which page needs to be displayed
		var page = parseInt(player.playlistPosition / itemsPerPage);
		player.getPlaylistItems(page * itemsPerPage, itemsPerPage);
		showPlaylistItemOnPageLoad = true;
		return;
	}

	// show the playlist if it is not already visible
	// delay the scroll request so scrolling works properly
	if (!showingPlaylist){
		showPlaylist();
		setTimeout(showCurrentPlaylistItem, 100);
		return;
	}

	var playlistItem = document.getElementById('playlist_' + player.playlistPosition);
	if (playlistItem) {
		window.scrollTo(0, playlistItem.offsetTop);
	}
}

function addBrowseHistoryEntry()
{
	var entry = new BrowseHistoryEntry();
	entry.start = mediaLibStart;
	entry.search = showingMediaLibSearchResults;
	if (!entry.search)
	{
		entry.path = currentBrowsePath;
	}
	else
	{
		entry.path = currentSearchKeywords;
	}

	browseHistory.push(entry);
}

function navigateLibBack() {
	if (browseHistory.length < 1) {
		backToNowPlaying();
		return;
	}

	var entry = browseHistory.pop();
	if (!entry.search)
	{
		player.browseLibrary(entry.path, entry.start, itemsPerPage);
	}
	else
	{
		if (!showingMediaLibSearchResults)
		{
			showingMediaLibSearchResults = true;

			if (document.getElementById)
			{
				message = document.getElementById('libheadertitle');
				message.innerHTML = 'Search';
			}
			else
			{
				libheadertitle.innerHTML = 'Search';
			}
		}

		currentSearchKeywords = entry.path;
		player.searchLibrary(entry.path, entry.start, itemsPerPage);
	}
}

function onAlbumArtChanged(url) {
	if (url == "") {
		url = "images/no_artwork.jpg";
	}

	var artHTML = "";
	if (url != "") {
		artHTML = '<img id="artwork" src="' + url + '">';
	}

	var albumart = document.getElementById(albumArtCurrentName);
	albumart.innerHTML = artHTML;
}

function onAlbumArtLoaded() {
	var albumArt = document.getElementById(albumArtCurrentName);
	var albumArtNew = document.getElementById(albumArtNewName);

	// set up the animation
	if (isMobileSafari) {
		albumArt.style.webkitTransitionProperty = "left";
		albumArtNew.style.webkitTransitionProperty = "left";
		albumArt.style.webkitTransitionDuration = "0.4s";
		albumArtNew.style.webkitTransitionDuration = "0.4s";
	}

	albumArtCurrentName = albumArtNew.id;
	albumArtNewName = albumArt.id;

	//albumArtNew.style.display = "block";

	albumArt.style.left = "-320px";
	albumArtNew.style.left = "0px";
}

function onAlbumArtAnimationEnd() {
	var albumArtNew = document.getElementById(albumArtNewName);

	if (isMobileSafari) {
		albumArtNew.style.webkitTransitionProperty = "none";
	}

	//albumArtNew.style.display = "none";
	albumArtNew.style.left = "320px";
}

function onRatingChanged(rating) {
	var numStars = rating / 20;

	for (i=1; i <= 5; i++) {
		var starElem = document.getElementById("star" + i);
		if (starElem) {
			if (i <= numStars) {
				starElem.className = "starred";
			} else {
				starElem.className = "";
			}
		}
	}
}

function searchInput_keyUp()
{
	// ignore if not in search mode
	if (!showingMediaLibSearch) {
		return;
	}

	var searchField = document.getElementById('searchInput');

	if (searchField.value.length > 2)
	{
		currentSearchKeywords = searchField.value;
		player.searchLibrary(searchField.value, 0, itemsPerPage);
	}
	else
	{
		var results = document.getElementById('libcontents');
		results.innerHTML = "";
	}
}

function toggleControls() {
	var subcontrols = document.getElementById('subcontrols');

	hideAddressBar();

	if (!showingSubControls) {
		subcontrols.style.visibility = 'visible';
		showingSubControls = true;
	} else {
		subcontrols.style.visibility = 'hidden';
		showingSubControls = false;
	}
}

function handleBrowseLibrary(object, path) {
	// object.className = "on";
	hideAddressBar();

	if (showingMediaLib) {
		addBrowseHistoryEntry();
	}

	player.browseLibrary(path, 0, itemsPerPage);
}

function handlePlayItem(object, itemId) {
	backToNowPlaying();
	player.playItem(itemId);
}

function handlePlayViewFromItem(object, itemId) {
	backToNowPlaying();
	player.playItem(currentBrowsePath, itemId);
}

function handlePlaylistJump(object, itemId) {
	backToNowPlaying();
	player.setPlaylistPosition(itemId);
}

function handleRating(rating) {
	currentRating = rating;
	onRatingChanged(rating);
	player.setRating(rating);
}

function handleActionPlay() {
	player.playItem(currentActionItem);
	showMediaLibActions();
}

function handleActionAdd() {
	player.playlistAdd(currentActionItem);
	showMediaLibActions();
}

function handleSingleItemAdd(object, itemId) {
	object.className = "off";
	player.playlistAdd(itemId);
}

function handleSpeakerSelection(id) {
	if (!useMultipleSpeakers) {
		backToNowPlaying();
		player.selectSpeaker(id);
	} else {
		player.toggleSpeaker(id);
	}
}

function toggleUseMultipleSpeakers() {
	useMultipleSpeakers = !useMultipleSpeakers;

	var msgElem = document.getElementById('speakermsg');
	var btnElem = document.getElementById('multiplespeakerbtn');
	btnElem.setAttribute("enabled", useMultipleSpeakers);

	if (useMultipleSpeakers) {
		msgElem.innerHTML = "Tap on a speaker name to toggle its audio on or off.";
	} else {
		msgElem.innerHTML = "Tap on a speaker name to send all audio to that speaker.";
	}

	if (useMultipleSpeakers) {
		setCookie("useMultipleSpeakers", "true", 365);
	} else {
		deleteCookie("useMultipleSpeakers");
	}
}

function mediaLibPrevPage() {
	player.browseLibrary(currentBrowsePath, mediaLibStart - itemsPerPage, itemsPerPage, false);
}

function mediaLibNextPage() {
	player.browseLibrary(currentBrowsePath, mediaLibStart + itemsPerPage, itemsPerPage, false);
}

function mediaLibJumpToLetter(letter) {
	player.browseLibrary(currentBrowsePath, letter, itemsPerPage, false);

	if (showingMediaLibAlphaJump)
		showMediaLibAlphaJump();
}

function playlistPrevPage() {
	player.getPlaylistItems(playlistStart - itemsPerPage, itemsPerPage, false);
}

function playlistNextPage() {
	player.getPlaylistItems(playlistStart + itemsPerPage, itemsPerPage, false);
}

function setLocation(page, recordHistory)
{
	if (!isMobileSafari)
		return;

	if (recordHistory == null || recordHistory == true)
		navHistory.push(location.hash);

	navCurrent = page;
	location.hash = navCurrent;
}

function slideTo(pos)
{
	setTimeout(function()
    {
		var percent = 100;
        var slideTimer = setInterval(slide, 0);

        function slide()
        {
            percent -= slideSpeed;
            if (percent <= 0)
            {
                percent = 0;
                clearInterval(slideTimer);
            }

            var x = ((100-percent)/100) * pos;
            scrollTo(0, x);
        }
    }, 0);
}

function setCookie(name, value, days) {
	var expires = "";

	if (days) {
		var date = new Date();
		date.setTime(date.getTime()+(days*24*60*60*1000));
		expires = "; expires="+date.toGMTString();
	}

	document.cookie = name + "=" + value + expires + "; path=/";
}

function getCookie(name) {
	name += "=";
	var cookieArray = document.cookie.split(';');
	for(var i=0; i < cookieArray.length; i++) {
		var testCookie = cookieArray[i];

		while (testCookie.charAt(0)==' ') {
			testCookie = testCookie.substring(1, testCookie.length);
		}

		if (testCookie.indexOf(name) == 0)
			return testCookie.substring(name.length, testCookie.length);
	}
	return null;
}

function deleteCookie(name) {
	setCookie(name, "", -1);
}

/** Monitors the search input field for changes to support
search-as-you-type on the iPhone. Normally this would be
accomplished via an onkeyup handler, but the iPhone does
not support this in firmware 1.0 */
function searchInputMonitor() {
	if (!showingMediaLibSearch) {
		return;
	}

	var searchField = document.getElementById('searchInput');
	if (searchField.value != searchInput) {
		searchInput = searchField.value;

		if (searchInput.length > 2)
		{
			currentSearchKeywords = searchInput;
			player.searchLibrary(searchInput, 0, itemsPerPage);
		}
		else
		{
			var results = document.getElementById('libcontents');
			results.innerHTML = "";
		}
	}
}

function uiMonitor()
{
	// only perform this UI processing for Mobile Safari
	if (!isMobileSafari)
		return;

	if (location.hash != navCurrent)
	{
		// user pressed the back or forward button
		var page = location.hash.substr(navHash.length)
		if (page == navCurrent)
		{
			// browser uses a different representation for the same location, we can ignore it
			navCurrent = location.hash;
			return;
		}

		navCurrent = location.hash;

		// check to see if the user has navigated backwards
		if (navHistory.length > 0 && location.hash == navHistory[navHistory.length - 1])
		{
			navHistory.pop();
		}
		else
		{
			// the user has navigated to a different destinaton, using the forward button
			// or manual hash entry
			navHistory.push(location.hash);
		}

		if (page == 'Media')
		{
			if (!showingMediaLib)
				showMediaLib(false);
		}
		else if (page == 'Playlist')
		{
			if (!showingPlaylist)
				showPlaylist(false);
		}
		else if (page == 'Speakers')
		{
			if (!showingSpeakers)
				showSpeakers(false);
		}
		else
		{
			if (showingMediaLib)
			{
				showMediaLib(false);
			}
			else if (showingPlaylist)
			{
				showPlaylist(false);
			}
			else if (showingSpeakers)
			{
				showSpeakers(false);
			}
		}
	}
}

function writeLog(msg, e)
{
	if (!loggingEnabled)
	{
		return;
	}

	try
	{
		if (e)
		{
			if (e.lineNumber)
			{
				msg += ': ' + e.toString() + ' at line ' + e.lineNumber + ' of ' + e.fileName + '\nstack:\n' + e.stack;
			}
			else
			{
				msg += ': ' + e.toString();
			}
		}

		console.log(msg);
	}
	catch (loge)
	{
	}
}
